"""This file defines all module wide default values."""


# Default remote settings
DEF_PORT = 4444


# Available templates
DEF_TEMPLATES = [
    "reverse",
    "bind",
]


# Credits
DEF_BIN = "kusa"
DEF_NAME = "kusanagi"
DEF_DESC = (
    "Kusanagi is a bind and reverse shell payload generator with obfuscation and badchar support."
)
DEF_VERSION = "0.0.2"
DEF_AUTHOR = "cytopia"
DEF_GITHUB = "https://github.com/cytopia/kusanagi"


# List of common badchars
BADCHARS = [
    "!",
    "",
    "#",
    "$",
    "%",
    "&",
    "'",
    "(",
    ")",
    "*",
    "+",
    ",",
    "-",
    ".",
    "/",
    ":",
    ";",
    "<",
    "=",
    ">",
    "?",
    "@",
    "[",
    "\\",
    "]",
    "_",
    "`",
    "{",
    "|",
    "}",
    "~",
    '"',
]
